/* Uses the "Unlicense" license.  http://unlicense.org/ */

#include <bzlib.h>
#include "SDL.h"

int
bz2rwops_read(SDL_RWops *ctx, void *ptr, int size, int maxnum)
{
	BZFILE *bz;
	int bzerror;
	int i;

	bz = (BZFILE *)ctx->hidden.unknown.data1;

	for (i=0; i<maxnum; i++) {
		BZ2_bzRead(&bzerror, bz, ptr, size);
		if (bzerror != BZ_OK && bzerror != BZ_STREAM_END) {
			SDL_Error(SDL_EFREAD);
			return -1;
		}

		ptr += size;
	}

	return i;
}

int
bz2rwops_close(SDL_RWops *ctx)
{
	BZFILE *bz;
	int bzerror;

	bz = (BZFILE *)ctx->hidden.unknown.data1;

	BZ2_bzReadClose(&bzerror, bz);
	if (bzerror != BZ_OK) {
		SDL_SetError("Could not close Bzip2 file");
		return -1;
	}

	SDL_FreeRW(ctx);

	return 0;
}

SDL_RWops *
make_bzip2_rwops(FILE *f)
{
	SDL_RWops *ops;
	int bzerror;

	ops = SDL_AllocRW();
	if (!ops)
		return NULL;

	ops->seek = NULL;
	ops->read = bz2rwops_read;
	ops->write = NULL;
	ops->close = bz2rwops_close;

	ops->hidden.unknown.data1 = BZ2_bzReadOpen(&bzerror, f, 0, 0, NULL, 0);
	if (bzerror != BZ_OK) {
		SDL_FreeRW(ops);
		return NULL;
	}

	return ops;
}

